@@ -1,7 +1,7 @@
 /*
- * @(#) $Id: cf.c 317 2022-07-30 20:27:00Z leres $ (XSE)
+ * @(#) $Id: cf.c 319 2022-07-31 18:11:02Z leres $ (XSE)
  *
- * Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008, 2009, 2015, 2017, 2018, 2019, 2020
+ * Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008, 2009, 2015, 2017, 2018, 2019, 2020, 2022
  *	Craig Leres
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,7 +41,8 @@
 /* Forwards */
 static char *dstrmerge(char *, char *);
 static void freelist(struct list *);
-static struct list *makelist(char *);
+static int keysort(const void *, const void *);
+static struct list *makelist(char *, const char *);
 static int readcf(const char *, struct cf *, int);
 static void fixpath(char **);
 
@@ -53,7 +54,7 @@ readcf(const char *fn, struct cf *cf, int mustexist)
 	char *cp, *cp2, *prefixend, **pp;
 	struct timespec ts;
 	FILE *f, *f2;
-	struct section *sp;
+	struct section *sp, *sp2;
 	struct list **lpp;
 	char *cp3, *include, *favicon, *appendhead, *appendtail;
 	char buf[1024];
@@ -298,7 +299,7 @@ readcf(const char *fn, struct cf *cf, int mustexist)
 			if (*lpp != NULL)
 				freelist(*lpp);
 			n += suck2dot(f, &cp3, 0);
-			*lpp = makelist(cp3);
+			*lpp = makelist(cp3, cp2);
 			free(cp3);
 			continue;
 		}
@@ -323,7 +324,7 @@ readcf(const char *fn, struct cf *cf, int mustexist)
 				exit(1);
 			}
 			n += suck2dot(f, &cp3, 1);
-			*lpp = makelist(cp3);
+			*lpp = makelist(cp3, cp2);
 			free(cp3);
 			continue;
 		}
@@ -408,6 +409,7 @@ readcf(const char *fn, struct cf *cf, int mustexist)
 				    "%s: %s missing leading double quote (2)\n",
 				    prog, at);
 				free(sp);
+				sp = NULL;
 				++errors;
 				continue;
 			}
@@ -426,12 +428,35 @@ readcf(const char *fn, struct cf *cf, int mustexist)
 				    " trailing double quote (2)\n", prog, at);
 				++errors;
 				free(sp);
+				sp = NULL;
 				free(*pp);
 				*pp = NULL;
 				continue;
 			}
 			*cp2 = '\0';
 
+			/* Look for duplicates */
+			sp2 = cf->c_section;
+			for (; sp2 != NULL; sp2 = sp2->s_next) {
+				if (strcmp(sp->s_file, sp2->s_file) == 0) {
+					fprintf(stderr,
+					    "%s: %s Duplicate section \"%s\"\n",
+					     prog, at, sp->s_file);
+					++errors;
+					free(sp);
+					free(*pp);
+					*pp = NULL;
+					sp = NULL;
+					cp3 = NULL;
+					n += suck2dot(f, &cp3, 1);
+					free(cp3);
+					cp3 = NULL;
+					break;
+				}
+			}
+			if (sp == NULL)
+				continue;
+
 			sp->s_next = cf->c_section;
 			cf->c_section = sp;
 
@@ -611,8 +636,18 @@ freecf(struct cf *cf)
 	free(cf);
 }
 
+static int
+keysort(const void *a1, const void *a2)
+{
+	const struct list *lp1, *lp2;
+
+	lp1 = a1;
+	lp2 = a2;
+	return (strcoll(lp1->l_key, lp2->l_key));
+}
+
 static struct list *
-makelist(char *str)
+makelist(char *str, const char *what)
 {
 	int i, n;
 	char *cp, *cp2;
@@ -706,6 +741,20 @@ makelist(char *str)
 
 	}
 
+	/* Sort */
+	qsort(list, n, sizeof(*list), keysort);
+
+	/* Report duplicates */
+	for (lp = list, i = 0; i < n - 1; ++i, ++lp) {
+		if (strcmp(lp->l_key, lp[1].l_key) == 0) {
+			fprintf(stderr,
+			    "%s: warning: Duplicate \"%s\" entry: %s\n",
+			    prog, what, lp->l_key);
+			/* Suppress "unused" warning */
+			lp[1].l_flags |= L_REFERENCED;
+		}
+	}
+
 	return (list);
 }
 
