@@ -1,5 +1,5 @@
 #!/bin/sh
-# @(#) $Id: updatetimestamps.sh 313 2022-07-15 21:35:58Z leres $ (XSE)
+# @(#) $Id: updatetimestamps.sh 316 2022-07-15 22:57:40Z leres $ (XSE)
 #
 # updatetimestamps - image the timestamps section of a webindex.conf file
 #
@@ -69,7 +69,7 @@ for d in ${dirs}; do
 		continue
 	fi
 
-	gentimestamps ${d} > ${t1}
+	gentimestamps ${d} | sort > ${t1}
 
 	(cat ${t1}
 	echo ''
